import argparse

from . application import Application



def parse_args():
	parser = argparse.ArgumentParser(prog="Server", description="Manage server deployment")
	subparsers = parser.add_subparsers(dest="command")

	create_parser = subparsers.add_parser("create", help="Create a new server")
	create_parser.add_argument('repository', help="Repository URL")

	add_parser = subparsers.add_parser("add", help="Add a repository")
	add_parser.add_argument('path', help="Relative path to repository")

	list_parser = subparsers.add_parser("list", help="List repositories")

	update_parser = subparsers.add_parser("update", help="Update server repositories")

	start_parser = subparsers.add_parser("start", help="Start server")
	stop_parser = subparsers.add_parser("stop", help="Stop server")

	process_parser = subparsers.add_parser("process", help="Process event")
	process_parser.add_argument('path', help="Event path")

	args = parser.parse_args()
	if args.command is None:
		parser.print_help()
		return

	return args



def main():
	args = parse_args()
	if args is None:
		return

	Application.Run(args)

if __name__ == '__main__':
	main()



import yaml
import pathlib



class Configuration:

	ROOT = pathlib.Path('~/.config/Server').expanduser().resolve()

	@classmethod
	def Initialize(self):
		self.first_run = not self.ROOT.exists()
		if not self.first_run:
			return

		self.ROOT.mkdir(parents=True, exist_ok=True)

	def __init__(self):
		self.path = self.ROOT / 'configuration.yaml'
		self.changed = False
		if self.first_run:
			self.store = {}
		else:
			self.store = self.Read()

	def __setitem__(self, *args, **kwargs):
		self.changed = True
		return self.store.__setitem__(*args, **kwargs)

	def __getitem__(self, *args, **kwargs):
		return self.store.__setitem__(*args, **kwargs)

	def __delitem__(self, *args, **kwargs):
		self.changed = True
		return self.store.__setitem__(*args, **kwargs)

	def get(self, *args, **kwargs):
		return self.store.get(*args, **kwargs)

	def Read(self):
		if not self.path.exists():
			return {}

		with open(self.path, "r") as f:
			return yaml.load(f, Loader=yaml.CSafeLoader)

	def Write(self):
		if not self.changed:
			return

		with open(self.path, "w") as f:
			yaml.dump(self.store, f, default_flow_style=False, Dumper=yaml.CSafeDumper)



import sys
import click
import socket
import pathlib
import logging
import importlib

from Git import Git, Repositories, Repository
from . configuration import Configuration



class Application:

	ROOT = pathlib.Path('~/source').expanduser().resolve()

	@classmethod
	def Initialize(self):
		if self.ROOT.exists():
			return

		self.ROOT.mkdir(parents=True, exist_ok=True)

	@classmethod
	def Run(self, args):
		self.Initialize()
		Configuration.Initialize()
		configuration = Configuration()

		application = self(configuration, args)
		status = application.run()

		if status:
			configuration.Write()

	def __init__(self, configuration, args):
		self.configuration = configuration
		self.command = args.command
		self.args = args

	def run(self):
		function = {
			'create': self.Create,
			'add': self.Add,
			'list': self.List,
			'start': self.Start,
			'stop': self.Stop,
			'update': self.Update,
			'process': self.Process,
		}

		fn = function[self.command]
		return fn()

	def hostname(self):
		return socket.gethostname()

	def Create(self):
		hostname = self.hostname()
		repository = self.args.repository

		status = []
		for r in ("webhook", "router", "certbot"):
			url = f'server/{hostname}/{r}'
			done = self.clone(repository, url, errors=False)
			if not done:
				click.echo("Failed " + click.style(f"{url}", fg="red"))
			status.append(done)

		if any(status):
			self.configuration['repository'] = repository
			return True

		return False

	def Add(self):
		repository = self.configuration.get('repository', None)
		path = self.args.path
		if repository is None:
			logging.error("No server deployment found")
			return False

		if (self.ROOT / path).exists():
			logging.error(f'Error: {path} exists')
			return False

		status = self.clone(repository, path, errors=False)
		if status:
			message = "Cloned " + click.style(f"{path}", fg='green')
		else:
			message = "Could not clone " + click.style(f"{path}", fg='red')

		click.echo(message)
		return status

	def Update(self):
		Git.Initialize(self.ROOT)
		for repository in Repositories():
			try:
				repository.pull()
				message = click.style('updated', fg='green')
			except Exception as e:
				message = click.style(f'{e}', fg='red')

			click.echo(f'{repository.path} ' + message)

	def List(self):
		Git.Initialize(self.ROOT)
		for repository in Repositories():
			click.echo(f'{repository.path}')

	def Start(self):
		Git.Initialize(self.ROOT, cached=False)

	def Stop(self):
		Git.Initialize(self.ROOT)

	def clone(self, repository, url, errors=True):
		path = self.ROOT / url
		link = f'{repository}/{url}.git'
		try:
			Repository.Clone(link, path)
			return True
		except Exception:
			if errors:
				raise
			return False

	def handler(self):
		path = self.ROOT / 'webhook'
		if not path.exists():
			return

		if len(sys.path) == 0 or sys.path[0] != path:
			sys.path.insert(0, path)

		try:
			return importlib.import_module('Handler')
		except ModuleNotFoundError:
			return

	def Process(self):
		handler = self.handler()
		if handler is None:
			logging.error("Handler could not be imported")
			return

		print(handler)
